#include <stdio.h>
#include <string.h>
#include <math.h>

// complementary or sequence string
#define ROOT 0
#define CMPL 1
#define SEQ 2

#define TWOPI (2*3.14)

#define UNIT 1.0
#define R 0.5
#define CIRCLESTYLE "fill=\"#FFFFFF\" stroke=\"#000000\" stroke-width=\"0.01\""
#define LINESTYLE "fill=\"#FFFFFF\" stroke=\"#000000\" stroke-width=\"0.1\""

typedef struct node {
  struct node* child;
  struct node* next;
  char* seq;
  int type;
  int x1;
  int y1;
  int x2;
  int y2;
} node;

void svghead() {
  printf("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n"
	 "<svg width=\"30cm\" height=\"60cm\" viewBox=\"0 0 30 60\"\n"
	 "\txmlns=\"http://www.w3.org/2000/svg\"\n"
	 "\txmlns:xlink=\"http://www.w3.org/1999/xlink\">\n"); 
}

void svgtail() {
  printf("</svg>\n");
}

void svgcircle(node* n) {
  printf("<circle cx=\"%f\" cy=\"%f\" r=\"%f\" " CIRCLESTYLE "/>\n", 
	 n->x1*UNIT, n->y1*UNIT, R);
}

void svgline(node* n) {
  printf("<line x1=\"%f\" y1=\"%f\" x2=\"%f\" y2=\"%f\" " LINESTYLE "/>\n", 
	 n->x1*UNIT, n->y1*UNIT, n->x2*UNIT, n->y2*UNIT);
}


node* newnode(int type, char* seq) {
  node* n = (node*)malloc(sizeof(node));
  n->type = type;
  if (seq != NULL) {
    n->seq = strdup(seq);
  }
  n->child = NULL;
  n->next = NULL;
  n->x1 = 0;
  n->y1 = 0;
  n->x2 = 0;
  n->y2 = 0;
  return n;
}

void disposenode(node* n) {
  free(n->seq);
  if (n->child != NULL) {
    disposenode(n->child);
  }

  if (n->next != NULL) {
    disposenode(n->next);
  }
}

void addnext(node* list, node* toadd) {
  node* tmp = list;
  while (tmp->next != NULL) {
    tmp = tmp->next;
  }

  tmp->next = toadd;
}

void addchild(node* parent, node* child) {
  if (parent->child == NULL) {
    parent->child = child;
  } else {
    addnext(parent->child, child);
  }
}

void listsetxy(node* list, int x, int y) {
  node* tmp = list;
  while (tmp != NULL) {
    tmp->x1 = x;
    tmp->y1 = y;
    tmp = tmp->next;
  }  
}

int nodelist_len(node* n) {
  node* tmp = n;
  int len = 0;

  while (tmp != NULL) {
    tmp = tmp->next;
    len++;
  }

  return len;
}

int seqlen(node* n) {
  switch (n->type) {
  case (SEQ) : return strlen(n->seq);
  case (CMPL) : return 2;
  default: return 0;
  }
}

int nodelist_seqlen(node* n) {
  node* tmp = n;
  int len = 0;

  while (tmp != NULL) {
    len += seqlen(tmp);

    tmp = tmp->next;
  }

  return len;
}

void compute_dest(node* n, int part, int whole) {
  double angle = 1.0*part/whole * TWOPI;
  double len = strlen(n->seq);

  n->x2 = n->x1 + len* sin(angle);
  n->y2 = n->y1 + len* cos(angle);
}

void walktree(node* root) {
  int seqslen;
  int len;
  int currlen = 0;
  node* tmp;

  switch (root->type) {
  case (SEQ):
  case (ROOT):
    if (root->child != NULL) {
      seqslen = nodelist_seqlen(root->child);
      len = nodelist_len(root->child);

      if (root->type == ROOT) {
	len--;
      }

    //    printf("circle (%dx%d). %d seqs of length: %d\n", 
    //	   root->x1, root->y1, len, seqslen);
    
      tmp = root->child;
      while (tmp != NULL) {
	currlen += seqlen(tmp);

	if (tmp->type == CMPL) {
	  
	  tmp->x1 = root->x1;
	  tmp->y1 = root->y1;
	  
	  compute_dest(tmp, currlen, seqslen);
	  
	  walktree(tmp);
	}
	tmp = tmp->next;
      }
    }

    svgcircle(root);

    break;

  case CMPL:
    len = strlen(root->seq);
    
    svgline(root);
    //printf("line of len %d (%dx%d-%dx%d)\n", len, root->x1, root->y1, root->x2, root->y2);

    listsetxy(root->child, root->x2, root->y2);

    if (root->child != NULL) {
      walktree(root->child);
    }
  }
}

void printnode(node* n) {
  printf("node %X: %d, seq: %s, child: %X, next: %X\n", n, n->type, n->seq, n->child, n->next);
}

int main() {
  node* root = NULL;
  node* tmp = NULL;

  root = newnode(ROOT, NULL);

  root->x1 = 5;
  root->y1 = 4;
  
  addchild(root, newnode(SEQ, "acug"));

  tmp = newnode(CMPL, "gcaugg");
  addchild(root, tmp);

  addchild(tmp, newnode(SEQ, "gcau"));

  tmp = newnode(CMPL, "gcaugg");
  addchild(root, tmp);  

  addchild(tmp, newnode(SEQ, "gcau"));

  svghead();
  walktree(root);
  svgtail();

  return 0;
}
