#include <stdio.h>
#include <string.h>
#include <math.h>

// complementary or sequence string
#define ROOT 0
#define CMPL 1
#define SEQ 2

#define TWOPI (2*3.14)

#define UNIT 1.0
#define RUNIT 0.1
#define R 0.5
#define WIDTH 30
#define HEIGHT 60
#define CIRCLESTYLE "fill=\"#FFFFFF\" stroke=\"#000000\" stroke-width=\"0.1\""
#define CIRCLESTYLE2 "fill=\"#FFFFFF\" stroke=\"#FFFFFF\" stroke-width=\"0\""
#define LINESTYLE "fill=\"#FFFFFF\" stroke=\"#000000\" stroke-width=\"0.3\""
#define LINESTYLE2 "fill=\"#FFFFFF\" stroke=\"#FFFFFF\" stroke-width=\"0.2\""

typedef struct node {
  struct node* child;
  struct node* next;
  char* seq;
  int type;
  int x1;
  int y1;
  int x2;
  int y2;
  int r;
} node;

void svghead() {
  printf("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n"
	 "<svg width=\"%dcm\" height=\"%dcm\" viewBox=\"0 0 %d %d\"\n"
	 "\txmlns=\"http://www.w3.org/2000/svg\"\n"
	 "\txmlns:xlink=\"http://www.w3.org/1999/xlink\">\n",
	 WIDTH, HEIGHT, WIDTH, HEIGHT); 
}

void svgtail() {
  printf("</svg>\n");
}

void svgcircle(node* n) {
  printf("<circle cx=\"%f\" cy=\"%f\" r=\"%f\" " CIRCLESTYLE "/>\n", 
	 n->x1*UNIT, n->y1*UNIT, n->r*RUNIT);
}

void svgcircle_invisible(node* n) {
  printf("<circle cx=\"%f\" cy=\"%f\" r=\"%f\" " CIRCLESTYLE2 "/>\n", 
	 n->x1*UNIT, n->y1*UNIT, n->r*RUNIT);
}

void svgline(node* n) {
  printf("<line x1=\"%f\" y1=\"%f\" x2=\"%f\" y2=\"%f\" " LINESTYLE "/>\n", 
	 n->x1*UNIT, n->y1*UNIT, n->x2*UNIT, n->y2*UNIT);
  printf("<line x1=\"%f\" y1=\"%f\" x2=\"%f\" y2=\"%f\" " LINESTYLE2 "/>\n", 
	 n->x1*UNIT, n->y1*UNIT, n->x2*UNIT, n->y2*UNIT);
}


node* newnode(int type, char* seq) {
  node* n = (node*)malloc(sizeof(node));
  n->type = type;
  if (seq != NULL) {
    n->seq = strdup(seq);
  }
  n->child = NULL;
  n->next = NULL;
  n->x1 = 0;
  n->y1 = 0;
  n->x2 = 0;
  n->y2 = 0;
  n->r = 0;
  return n;
}

void disposenode(node* n) {
  free(n->seq);
  if (n->child != NULL) {
    disposenode(n->child);
  }

  if (n->next != NULL) {
    disposenode(n->next);
  }
}

void addnext(node* list, node* toadd) {
  node* tmp = list;
  while (tmp->next != NULL) {
    tmp = tmp->next;
  }

  tmp->next = toadd;
}

void addchild(node* parent, node* child) {
  if (parent->child == NULL) {
    parent->child = child;
  } else {
    addnext(parent->child, child);
  }

  fprintf(stderr, "addchild parent %X(%d) child %X(%d)\n", parent, parent->type, child, child->type);
}

void listsetxy(node* list, int x, int y) {
  node* tmp = list;
  while (tmp != NULL) {
    tmp->x1 = x;
    tmp->y1 = y;
    tmp = tmp->next;
  }  
}

int nodelist_len(node* n) {
  node* tmp = n;
  int len = 0;

  while (tmp != NULL) {
    tmp = tmp->next;
    len++;
  }

  return len;
}

int seqlen(node* n) {
  switch (n->type) {
  case (SEQ) : return strlen(n->seq);
  case (CMPL) : return 2;
  default: return 0;
  }
}

int nodelist_seqlen(node* n) {
  node* tmp = n;
  int len = 0;

  while (tmp != NULL) {
    len += seqlen(tmp);

    tmp = tmp->next;
  }

  return len;
}

void compute_dest(node* n, int part, int whole) {
  int ratio = part; //(part + whole/2) % whole;
  //  double angle = (0.25 + 1.0*ratio/(whole*2)) * TWOPI;
  double angle = 1.0*ratio/whole * TWOPI;
  double len = strlen(n->seq);
  len = len*len/4;

  n->x2 = n->x1 + len* sin(angle);
  n->y2 = n->y1 - len* cos(angle);
}

void walktree(node* root) {
  int seqslen = 0;
  int len = 0;
  int currlen = 0;
  node* tmp;

  fprintf(stderr, "node %X(%d)\n", root, root->type);

  seqslen = nodelist_seqlen(root->child) + 2;
  len = nodelist_len(root->child) - 1;
  root->r = seqslen/3;

  if (seqslen > 0) {
    svgcircle(root);  
  }

  tmp = root->child;
  int l = 1;
  while (tmp != NULL) {
    currlen += seqlen(tmp);

    tmp->x1 = root->x1;
    tmp->y1 = root->y1;    

    if (tmp->type == CMPL) {
      
      compute_dest(tmp, currlen, seqslen);
      //compute_dest(tmp, l, len);
      svgline(tmp);
      tmp->x1 = tmp->x2;
      tmp->y1 = tmp->y2;

      walktree(tmp);
    }
    tmp = tmp->next;
    
    l++;
  }

  if (seqslen > 0) {
    svgcircle_invisible(root);  
  }

}

void printnode(node* n) {
  printf("node %X: %d, seq: %s, child: %X, next: %X\n", n, n->type, n->seq, n->child, n->next);
}

